import { ListenerRegistry } from "./util/ListenerRegistry";
/**
 * A base interface for entities, which are just normal JavaScript objects with
 * any number of properties, each of which represents a single component. When
 * using hmecs, it is recommended to create your own type representing your
 * game's entities, and pass that to `createECS` for full typing support.
 */
export interface IEntity {
    id?: number;
}
export declare type UntypedEntity = {
    [components: string]: ComponentData;
} & IEntity;
/**
 * Component names are just strings/object property names.
 */
export declare type ComponentName<T extends IEntity> = keyof T;
/**
 * The data of a component can be literally anything. Good times!
 */
export declare type ComponentData = any;
/**
 * hmecs lets you define archetypes of entities. For each archetype,
 * it will automatically create an index; entities matching this archetype
 * will automatically be added to the index, entities no longer matching
 * the archetype will automatically be removed.
 */
export declare type Archetype<T extends IEntity> = ComponentName<T>[];
export declare type ArchetypeIndex<T extends IEntity> = Map<Archetype<T>, T[]>;
declare type ImmediateAPI<T> = {
    addEntity: (entity: T) => T;
    removeEntity: (entity: T) => void;
    addComponent: <U extends ComponentName<T>>(entity: T, name: U, data: T[U]) => void;
    removeComponent: (entity: T, ...names: ComponentName<T>[]) => void;
};
declare type Listeners<T> = {
    listeners: {
        archetypeChanged: Map<Archetype<T>, ListenerRegistry>;
    };
};
export declare type ECS<T extends IEntity> = {
    entities: T[];
    immediately: ImmediateAPI<T>;
    createArchetype: (...components: ComponentName<T>[]) => Archetype<T>;
    get: (archetype: Archetype<T>) => T[];
    getWith: (...components: ComponentName<T>[]) => T[];
    flush: () => void;
} & ImmediateAPI<T> & Listeners<T>;
export declare function createECS<T extends IEntity = UntypedEntity>(): ECS<T>;
export {};
